==> mcss-0.0.31.tar.gz: mcss-0.0.31/mcss/MulticompartmentGillespieCellularPotts.cpp <==
// multicompartment gillespie cellular potts class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "MulticompartmentGillespieCellularPotts.h"

#include <limits>
#include <ecsb/ecsb.h>

using namespace Simulation;
using namespace ECSB;

// constructor
MulticompartmentGillespieCellularPotts:: \
		MulticompartmentGillespieCellularPotts(const unsigned long seed) \
		: SimulationAlgorithm("Multicompartment Gillespie Cellular Potts", seed)
{
	waiting_times_hash = NULL;
	potts_timestep = 1.0E-5;
}

// destructor
MulticompartmentGillespieCellularPotts:: \
		~MulticompartmentGillespieCellularPotts()
{
	if(waiting_times_hash)
		free(waiting_times_hash);
}

// initialise algorithm
int MulticompartmentGillespieCellularPotts::preprocess(void)
{
	register unsigned long i;
	register double tautmp;
	Compartment *compartment;

	// allocate memory for hash of global waiting times
	if(!waiting_times_hash)
		if(!(waiting_times_hash = (double *) calloc(num_compartments, \
				sizeof(double)))) {
			cerr << "error: couldn't allocate memory for compartments\n";
			exit(EXIT_FAILURE);
		}
	tau0global = 0.0;

	// set pointer to p system lattice
	lattice = psystem->getLattice();

	// calculate tau and mu for each compartment
	for(i = 0; i < num_compartments; i++) {
		compartment = psystem->getCompartment(i);
		psystem->execute(i);
		tautmp = compartment->getTimeToNextReaction();
		// add tau to map of compartment waiting times
		if(tautmp != 0.0) {
			waiting_times.insert(waiting_time_entry(tautmp, i));
			waiting_times_hash[i] = tautmp;
		} else
			waiting_times_hash[i] = 0.0;
	}

	// finish if no more reactions to execute
 	if(waiting_times.empty()) {
 		tau0 = mu0 = i0 = rs0 = 0;
 		return 0;
 	}

	// get compartment with smallest waiting time
	waiting_times_head = waiting_times.begin();
	tau0global = waiting_times_head->first;
	i0 = waiting_times_head->second;
	compartment = psystem->getCompartment(i0);
	tau0 = compartment->getTimeToNextReaction();
	mu0 = compartment->getNextReaction();
	rs0 = compartment->getNextRuleSet();
	
	return 1;
}

// perform one iteration of algorithm for all compartments
// using multicompartment gillespie cellular potts algorithm
int MulticompartmentGillespieCellularPotts::execute(void)
{
	register unsigned long it;
	register double tautmp, energy;
	register Compartment *compartment;
	register LatticeSite *site, *neighbour;
	register bool flip;

	// perform gillespie
	// update simulation time
	setTime(tau0global);

	// perform reaction in selected compartment
	compartment = psystem->getCompartment(i0);
	psystem->executeRule(rs0, mu0, i0);

	// run gillespie to recalculate next reaction in selected compartment
	psystem->execute(i0);

	// remove old waiting time from map of compartment waiting times
	waiting_times.erase(waiting_times_head);
	// add new waiting time to map of compartment waiting times
	tautmp = compartment->getTimeToNextReaction() + tau0global;
	if(tautmp != tau0global) {
		waiting_times.insert(waiting_time_entry(tautmp, i0));
		waiting_times_hash[i0] = tautmp;
	}

	// update target compartment if channel
	if(compartment->isChannel(rs0, mu0)) {
		it = compartment->getRuleTarget(rs0, mu0);
		// remove waiting time from map of compartment waiting times
		pair<waiting_time_iterator, waiting_time_iterator> range = \
				waiting_times.equal_range(waiting_times_hash[it]);
		for(waiting_time_iterator p = range.first; p != range.second; ++p)
			if(p->second == it) {
				waiting_times.erase(p);
				break;
			}
		// run gillespie to recalculate next reaction in target compartment
		psystem->execute(it);
		// add new waiting time to map of compartment waiting times
		compartment = psystem->getCompartment(it);
		tautmp = compartment->getTimeToNextReaction() + tau0global;
		if(tautmp != tau0global) {
			waiting_times.insert(waiting_time_entry(tautmp, it));
			waiting_times_hash[it] = tautmp;
		}
	}

	// perform cellular potts update of lattice
	// select random lattice site
	do {
		// select random site with at least one alien neighbour
		site = lattice->getRandomSite();
	} while (!lattice->calculateAlienNeighbours(site));

	// choose random neighbour
	neighbour = lattice->selectAlienNeighbour();

	// calculate energy to flip site spin to neighbouring site spin
	energy = lattice->calculateFlipEnergy(site, neighbour);

	// determine if energy is favourable to flip site spin
	flip = false;
	if(energy <= 0.0)
		flip = true;
	else
		if(randomNumber() < exp(-1.0 * energy / lattice->getTemperature()))
			flip = true;

	// if flipping thne update volumes + waiting times then flip
	if(flip) {
		// add time taken for flip to simualtion time
		tau0global += potts_timestep;

		// adjust compartment volumes if flipped
		// handle site compartment
		if((it = site->spin)) {
			// site is not environment
			// get compartment associated with site
			compartment = psystem->getCompartment(--it);
			// decrease volume
			compartment->decreaseVolume(lattice->getSiteVolume());
			// remove waiting time from map of compartment waiting times
			pair<waiting_time_iterator, waiting_time_iterator> range = \
					waiting_times.equal_range(waiting_times_hash[it]);
			for(waiting_time_iterator p = range.first; p != range.second; ++p)
				if(p->second == it) {
					waiting_times.erase(p);
					break;
				}
			// run gillespie to recalculate next reaction in compartment
			psystem->execute(it);
			// add new waiting time to map of compartment waiting times
			tautmp = compartment->getTimeToNextReaction() + tau0global;
			if(tautmp != tau0global) {
				waiting_times.insert(waiting_time_entry(tautmp, it));
				waiting_times_hash[it] = tautmp;
			}
		}
		// handle neighbour compartment
		if((it = neighbour->spin)) {
			// neighbour is not environment
			// get compartment associated with neighbour
			compartment = psystem->getCompartment(--it);
			// increase volume
			compartment->increaseVolume(lattice->getSiteVolume());
			// remove waiting time from map of compartment waiting times
			pair<waiting_time_iterator, waiting_time_iterator> range = \
					waiting_times.equal_range(waiting_times_hash[it]);
			for(waiting_time_iterator p = range.first; p != range.second; ++p)
				if(p->second == it) {
					waiting_times.erase(p);
					break;
				}
			// run gillespie to recalculate next reaction in compartment
			psystem->execute(it);
			// add new waiting time to map of compartment waiting times
			tautmp = compartment->getTimeToNextReaction() + tau0global;
			if(tautmp != tau0global) {
				waiting_times.insert(waiting_time_entry(tautmp, it));
				waiting_times_hash[it] = tautmp;
			}
		}

		// flip site spin to neighbouring site spin
		lattice->flipSite(site, neighbour);
	}

	// finish if no more reactions to execute
// 	if(waiting_times.empty()) {
// 		tau0 = mu0 = i0 = rs0 = 0;
// 		return 0;
// 	}

	// get compartment with smallest waiting time
	waiting_times_head = waiting_times.begin();
	tau0global = waiting_times_head->first;
	i0 = waiting_times_head->second;
	compartment = psystem->getCompartment(i0);
	tau0 = compartment->getTimeToNextReaction();
	mu0 = compartment->getNextReaction();
	rs0 = compartment->getNextRuleSet();

	return 1;
}

// perform one iteration of algorithm for one compartment
// using gillespie direct method
void MulticompartmentGillespieCellularPotts::execute(unsigned long c)
{
	register unsigned long i, mu0, rs0;
	register double a0, tau0, r1, r2, am;
	register Compartment *compartment;

	// don't run for compartments with no rules
//	if(!compartment->getNumRules())
//		return;

	compartment = psystem->getCompartment(c);
	
	// calculate propensities
	a0 = 0.0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(i = 0; i < compartment->getNumRules(rs0); i++)
			a0 += calculatePropensity(rs0, i, c);

	// finish if zero level
	if(a0 == 0.0) {
		compartment->setTimeToNextReaction(0.0);
		compartment->setNextReaction(0);
		compartment->setNextRuleSet(0);
		return;
	}

	// generate time to next reaction
	r1 = randomNumber();
	tau0 = -1.0 * log(r1) / a0;

	// select next reaction
	r2 = randomNumber();
	am = r2 * a0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(mu0 = 0; mu0 < compartment->getNumRules(rs0); mu0++) {
			am -= compartment->getPropensity(rs0, mu0);
			if(am <= 0.0)
				goto done;
		}
		
	done:
		compartment->setTimeToNextReaction(tau0);
		compartment->setNextReaction(mu0);
		compartment->setNextRuleSet(rs0);
}

==> mcss-0.0.39.tar.gz: mcss-0.0.39/mcss/MulticompartmentGillespieCellularPotts.cpp <==
// multicompartment gillespie cellular potts class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "MulticompartmentGillespieCellularPotts.h"

#include <limits>
#include <ecsb/ecsb.h>
#include <gsl/gsl_sf_log.h>

using namespace Simulation;
using namespace ECSB;

// constructor
MulticompartmentGillespieCellularPotts:: \
		MulticompartmentGillespieCellularPotts(const unsigned long seed) \
		: SimulationAlgorithm("Multicompartment Gillespie Cellular Potts", seed)
{
	waiting_times_hash = NULL;
	gillespie_steps = 0;
}

// destructor
MulticompartmentGillespieCellularPotts:: \
		~MulticompartmentGillespieCellularPotts()
{
	if(waiting_times_hash)
		free(waiting_times_hash);
}

// initialise algorithm
int MulticompartmentGillespieCellularPotts::preprocess(void)
{
	register unsigned long i;
	register double tautmp;
	Compartment *compartment;

	// allocate memory for hash of global waiting times
	if(!waiting_times_hash) {
		if(!(waiting_times_hash = (double *) calloc(num_compartments, \
				sizeof(double)))) {
			cerr << "error: couldn't allocate memory for compartments\n";
			exit(EXIT_FAILURE);
		}
	} else
		waiting_times.clear();

	// set pointer to p system lattice
	lattice = psystem->getLattice();

	// set reaction counter
	gillespie_steps = 0;

	// calculate tau and mu for each compartment
	for(i = 0; i < num_compartments; i++) {
		compartment = psystem->getCompartment(i);
		psystem->execute(i);
		tautmp = compartment->getTimeToNextReaction();
		// add tau to map of compartment waiting times
		if(tautmp != 0.0) {
			tautmp += tau0global;
			waiting_times.insert(waiting_time_entry(tautmp, i));
			waiting_times_hash[i] = tautmp;
		} else
			waiting_times_hash[i] = 0.0;
	}

	// finish if no more reactions to execute
 	if(waiting_times.empty()) {
 		tau0 = mu0 = i0 = rs0 = 0;
 		return 0;
 	}

	// get compartment with smallest waiting time
	waiting_times_head = waiting_times.begin();
	tau0global = waiting_times_head->first;
	i0 = waiting_times_head->second;
	compartment = psystem->getCompartment(i0);
	tau0 = compartment->getTimeToNextReaction();
	mu0 = compartment->getNextReaction();
	rs0 = compartment->getNextRuleSet();

	return 1;
}

// perform one iteration of algorithm for all compartments
// using multicompartment gillespie cellular potts algorithm
int MulticompartmentGillespieCellularPotts::execute(void)
{
	register unsigned long it;
	register double tautmp, energy;
	register Compartment *compartment;
	register LatticeSite *site, *neighbour;
	register bool flip;

	// perform gillespie
	// update simulation time
	setTime(tau0global);

	// perform reaction in selected compartment
	compartment = psystem->getCompartment(i0);
	psystem->executeRule(rs0, mu0, i0);

	// run gillespie to recalculate next reaction in selected compartment
	psystem->execute(i0);

	// remove old waiting time from map of compartment waiting times
	waiting_times.erase(waiting_times_head);
	// add new waiting time to map of compartment waiting times
	tautmp = compartment->getTimeToNextReaction();
	if(tautmp != 0.0) {
		tautmp += tau0global;
		waiting_times.insert(waiting_time_entry(tautmp, i0));
		waiting_times_hash[i0] = tautmp;
	}

	// update target compartment if channel
	if(compartment->isChannel(rs0, mu0)) {
		it = compartment->getRuleTarget(rs0, mu0);
		// remove waiting time from map of compartment waiting times
		pair<waiting_time_iterator, waiting_time_iterator> range = \
				waiting_times.equal_range(waiting_times_hash[it]);
		for(waiting_time_iterator p = range.first; p != range.second; ++p)
			if(p->second == it) {
				waiting_times.erase(p);
				break;
			}
		// run gillespie to recalculate next reaction in target compartment
		psystem->execute(it);
		// add new waiting time to map of compartment waiting times
		compartment = psystem->getCompartment(it);
		tautmp = compartment->getTimeToNextReaction();
		if(tautmp != 0.0) {
			tautmp += tau0global;
			waiting_times.insert(waiting_time_entry(tautmp, it));
			waiting_times_hash[it] = tautmp;
		}
	}

	if(!static_lattice && ++gillespie_steps >= potts_rate) {
		// perform cellular potts update of lattice
		// select random lattice site
		do {
			// select random site with at least one alien neighbour
			site = lattice->getRandomSite();
		} while (!lattice->calculateAlienNeighbours(site));

		// choose random neighbour
		neighbour = lattice->selectAlienNeighbour();

		// calculate energy to flip site spin to neighbouring site spin
		energy = lattice->calculateFlipEnergy(site, neighbour);

		// determine if energy is favourable to flip site spin
		flip = false;
		if(energy <= 0.0)
			flip = true;
		else
			if(randomNumber() < exp(-1.0 * energy / lattice->getTemperature()))
				flip = true;

		// if flipping then update volumes + waiting times then flip
		if(flip) {
			// adjust compartment volumes if flipped
			// handle site compartment
			if((it = site->spin)) {
				// site is not environment
				// get compartment associated with site
				compartment = psystem->getCompartment(--it);
				// decrease volume
				compartment->decreaseVolume(lattice->getSiteVolume());
				// remove waiting time from map of compartment waiting times
				pair<waiting_time_iterator, waiting_time_iterator> range = \
						waiting_times.equal_range(waiting_times_hash[it]);
				for(waiting_time_iterator p = range.first; p != range.second; ++p)
					if(p->second == it) {
						waiting_times.erase(p);
						break;
					}
				// run gillespie to recalculate next reaction in compartment
				psystem->execute(it);
				// add new waiting time to map of compartment waiting times
				tautmp = compartment->getTimeToNextReaction();
				if(tautmp != 0.0) {
					tautmp += tau0global;
					waiting_times.insert(waiting_time_entry(tautmp, it));
					waiting_times_hash[it] = tautmp;
				}
			}
			// handle neighbour compartment
			if((it = neighbour->spin)) {
				// neighbour is not environment
				// get compartment associated with neighbour
				compartment = psystem->getCompartment(--it);
				// increase volume
				compartment->increaseVolume(lattice->getSiteVolume());
				// remove waiting time from map of compartment waiting times
				pair<waiting_time_iterator, waiting_time_iterator> range = \
						waiting_times.equal_range(waiting_times_hash[it]);
				for(waiting_time_iterator p = range.first; p != range.second; ++p)
					if(p->second == it) {
						waiting_times.erase(p);
						break;
					}
				// run gillespie to recalculate next reaction in compartment
				psystem->execute(it);
				// add new waiting time to map of compartment waiting times
				tautmp = compartment->getTimeToNextReaction();
				if(tautmp != 0.0) {
					tautmp += tau0global;
					waiting_times.insert(waiting_time_entry(tautmp, it));
					waiting_times_hash[it] = tautmp;
				}
			}

			// flip site spin to neighbouring site spin
			lattice->flipSite(site, neighbour);
		}

		gillespie_steps = 0;
	}

	// finish if no more reactions to execute
	if(waiting_times.empty()) {
		tau0 = mu0 = i0 = rs0 = 0;
		return 0;
	}

	// get compartment with smallest waiting time
	waiting_times_head = waiting_times.begin();
	tau0global = waiting_times_head->first;
	i0 = waiting_times_head->second;
	compartment = psystem->getCompartment(i0);
	tau0 = compartment->getTimeToNextReaction();
	mu0 = compartment->getNextReaction();
	rs0 = compartment->getNextRuleSet();

	return 1;
}

// perform one iteration of algorithm for one compartment
// using gillespie direct method
void MulticompartmentGillespieCellularPotts::execute(unsigned long c)
{
	register unsigned long i, mu0, rs0;
	register double a0, tau0, r1, r2, am;
	register Compartment *compartment;

	// don't run for compartments with no rules
//	if(!compartment->getNumRules())
//		return;

	compartment = psystem->getCompartment(c);

	// calculate propensities
	a0 = 0.0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(i = 0; i < compartment->getNumRules(rs0); i++)
			a0 += calculatePropensity(rs0, i, c);

	// finish if zero level
	if(a0 == 0.0) {
		compartment->setTimeToNextReaction(0.0);
		compartment->setNextReaction(0);
		compartment->setNextRuleSet(0);
		return;
	}

	// generate time to next reaction
	r1 = randomNumberPos();
	tau0 = -1.0 * gsl_sf_log(r1) / a0;

	// select next reaction
	r2 = randomNumberPos();
	am = r2 * a0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(mu0 = 0; mu0 < compartment->getNumRules(rs0); mu0++) {
			am -= compartment->getPropensity(rs0, mu0);
			if(am <= 0.0)
				goto done;
		}

	done:
		compartment->setTimeToNextReaction(tau0);
		compartment->setNextReaction(mu0);
		compartment->setNextRuleSet(rs0);
}
